// (c) Thomas Kamps, 2009
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <QMessageBox>
#include <QTableWidget>

#include "capitalism.h"
#include "ui_capitalism.h"

Capitalism::Capitalism(QTcpSocket* sok, QWidget *parent)
                       :QMainWindow(parent), ui(new Ui::Capitalism) {
	ui->setupUi(this);

	setClient(new MonopdClient(sok));

	//Tapify Docks
	tabifyDockWidget(ui->dockJail, ui->dockTurn);
//	tabifyDockWidget(ui->dockEstate, ui->dockJail);
	tabifyDockWidget(ui->dockTurn, ui->dockJail);
	tabifyDockWidget(ui->dockJail, ui->dockEstate);
	tabifyDockWidget(ui->dockEvents, ui->dockChat);

	//Set Table with games as central widget
	newGames();
	newGameConfig();
	newField();
	layout = new QStackedLayout();
	layout->setObjectName("Stack");
	layout->addWidget(games);
	layout->addWidget(gameConfig);
	layout->addWidget(field);
	layout->addWidget(client);
	ui->center->addLayout(layout);
	layout->setCurrentIndex(3);

	selectedField = -1;

	//Turn
	ui->bRoll->cmd = ".r";
	ui->bBuy->cmd = ".eb";
	ui->bAuction->cmd = ".ea";
	ui->bEndturn->cmd = ".E";
	connect(ui->bRoll, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuy, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuy, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bAuction, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bAuction, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bEndturn, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bEndturn, SIGNAL(clicked()), field, SLOT(clearButtons()));

	//Jail
	ui->bJailRoll->cmd = ".jr";
	ui->bCard->cmd = ".jc";
	ui->bPay->cmd = ".jp";
	connect(ui->bJailRoll, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bCard, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bPay, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));

	//Estate
	connect(ui->bSell, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuyHouse, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bSellHouse, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bMortage, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));

	//chat
	connect(ui->bSend, SIGNAL(clicked()), this, SLOT(sendMsg()));
	connect(ui->tfMess, SIGNAL(returnPressed()), this, SLOT(sendMsg()));

	//news
	connect(ui->dockChat, SIGNAL(visibilityChanged(bool)), this, SLOT(normalChatTitle(bool)));
	connect(ui->dockEvents, SIGNAL(visibilityChanged(bool)), this, SLOT(normalEventTitle(bool)));
}

Capitalism::~Capitalism() {
	delete ui;
}

MonopdGame* Capitalism::getGame() {
	return game;
}

void Capitalism::setGame(MonopdGame* g) {
	game = g;
	if (field == 0) return;
	field->setGame(g);
}

void Capitalism::roll() {
	if (client == 0) return;
	client->send(".r");
}

void Capitalism::buy() {
	if (client == 0) return;
	client->send(".eb");
}

void Capitalism::jailRoll() {
	if (client == 0) return;
	client->send(".jr");
}

void Capitalism::jailCard() {
	if (client == 0) return;
	client->send(".jc");
}

void Capitalism::jailPay() {
	if (client == 0) return;
	client->send(".jp");
}

void Capitalism::sendMsg() {
	if (client == 0) return;
	client->send(ui->tfMess->text());
	ui->tfMess->setText("");
}

void Capitalism::setSelectedField(MonopdField* f) {
	if ((f == 0) || (f->getGroup() == -1)) {
		selectedField = -1;
	} else {
		selectedField = f->getId();
	}
	updateEstateDock();
}

void Capitalism::updateEstateDock() {
	if (selectedField == -1) {
		ui->dockEstate->setWindowTitle("Estate");
		ui->dockEstate->setEnabled(false);
	} else {
		MonopdField* f = game->getField(selectedField);
		ui->dockEstate->setWindowTitle("Estate: "+f->getName());
		ui->dockEstate->setEnabled(f->getOwner() == client->getClientPlayerId());

		ui->bMortage->setEnabled(f->getCanToggleMortage());
		if (f->getMortaged()) ui->bMortage->setIcon(QIcon(":/Icons/Tools/Unmortage.png"));
		else ui->bMortage->setIcon(QIcon(":/Icons/Tools/Mortage.png"));
		ui->bMortage->cmd = ".em"+QString::number(selectedField);

		ui->bBuyHouse->setEnabled(f->getCanBuyHouse());
		ui->bBuyHouse->cmd = ".hb"+QString::number(selectedField);

		ui->bSellHouse->setEnabled(f->getCanSellHouse());
		ui->bSellHouse->cmd = ".hs"+QString::number(selectedField);

		ui->bSell->cmd = ".es"+QString::number(selectedField);
	}
}

void Capitalism::normalChatTitle(bool vis) {
	chatVisible = vis;
	if (!vis) return;
	ui->dockChat->setWindowTitle("Chat");
}

void Capitalism::normalEventTitle(bool vis) {
	eventVisible = vis;
	if (!vis) return;
	ui->dockEvents->setWindowTitle("Events");
}

void Capitalism::setClient(MonopdClient* c) {
	client = c;
	//c->connectToHost("localhost", 1234);
	connect(client, SIGNAL(connected()), this, SLOT(setCurGame()));

	connect(client, SIGNAL(fieldUpdate()), this, SLOT(drawField()));
	connect(client, SIGNAL(fieldUpdate()), this, SLOT(updateEstateDock()));
	connect(client, SIGNAL(playerAdded(Player*)), this, SLOT(addToList(Player*)));
	connect(client, SIGNAL(chatMessage(int, QString)), this, SLOT(showChat(int, QString)));
	connect(client, SIGNAL(serverMessage(QString, QString)), this, SLOT(showEvent(QString, QString)));
	connect(client, SIGNAL(playerUpdated()), this, SLOT(playerUpdate()));
	connect(client, SIGNAL(newGameList()), this, SLOT(updateGameList()));
	connect(client, SIGNAL(curGameChanged()), this, SLOT(setCurGame()));
	connect(client, SIGNAL(statusChanged(QString)), this, SLOT(setCurGame()));

	connect(client, SIGNAL(tradeUpdate(QString,int,int,int)), this, SLOT(tradeUpdate(QString,int,int,int)));
	connect(client, SIGNAL(tradePlayerUpdate(int,int,bool)), this, SLOT(tradePlayerUpdate(int,int,bool)));
	connect(client, SIGNAL(tradeEstateUpdate(int,int,int)), this, SLOT(tradeEstateUpdate(int,int,int)));
	connect(client, SIGNAL(tradeCardUpdate(int,int,int)), this, SLOT(tradeCardUpdate(int,int,int)));
	connect(client, SIGNAL(tradeMoneyUpdate(int,int,int,int)), this, SLOT(tradeMoneyUpdate(int,int,int,int)));
}

void Capitalism::playerUpdate() {
	Player* p = client->getPlayer(client->getClientPlayerId());
	if (p == 0) return;
	ui->bRoll->setEnabled(p->getCanRoll());
	ui->bAuction->setEnabled(p->getCanAuction());
	ui->bBuy->setEnabled(p->getCanBuyEstate());
	ui->dockJail->setEnabled(p->getInJail());
	ui->bCard->setEnabled(p->getCanUseCard());
	ui->bEndturn->setEnabled(p->getHasTurn() && !p->getInJail() && p->getCanBuyEstate() && !p->getCanAuction());
}

void Capitalism::updateGameList() {
	if (games == 0) return;
	if (client == 0) return;
	if (client->getCurGameId() != -1) return;
	games->clearContents();
	games->setColumnCount(4);
	games->setHorizontalHeaderLabels(QStringList("Name") << "Description" << "Players" << "joinCmd");
	games->setVerticalHeaderLabels(QStringList());

	games->setRowCount(client->getNumTemplates()+client->getNumGames());
	int r = 0;
	for (int i = 0; i < client->getNumGames(); i++) {
		QTableWidgetItem* it = new QTableWidgetItem(client->getGameFromList(i)->getName());
		bool cbj = client->getGameFromList(i)->getCanBeJoined();
		if (cbj) it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		else it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 0, it);

		it = new QTableWidgetItem(client->getGameFromList(i)->getDesc());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 1, it);

		it = new QTableWidgetItem(QString::number(client->getGameFromList(i)->getNumPlayer()));
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 2, it);

		it = new QTableWidgetItem(".gj"+QString::number(client->getGameFromList(i)->getGameId()));
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 3, it);

		r++;
	}

	for (int i = 0; i < client->getNumTemplates(); i++) {
		QTableWidgetItem* it = new QTableWidgetItem(client->getTemplateFromList(i)->getName());
		it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		it->setBackground(QBrush(QColor(192,255,192)));
		games->setItem(r, 0, it);

		it = new QTableWidgetItem(client->getTemplateFromList(i)->getDesc());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 1, it);

		it = new QTableWidgetItem("");
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 2, it);

		it = new QTableWidgetItem(".gn"+client->getTemplateFromList(i)->getType());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 3, it);

		r++;
	}
	games->hideColumn(3);
	games->resizeColumnsToContents();
}

void Capitalism::showChat(int pid, QString msg) {
	Player* p;
	if (client->getCurGameId() != -1) {
		p = game->getPlayer(pid);
	}
	else p = client->getPlayer(pid);
	if (p == 0) {
		ui->chat->append("<span style=\"color:#808080\">Unknown Player:</span> "+msg);
	} else {
		ui->chat->append("<span style=\"color:#"+p->getChatColor()+"\">"+p->getName()+":</span> "+msg);
	}
	if (!chatVisible) {
		ui->dockChat->setWindowTitle("*Chat*");
	}
}

void Capitalism::showEvent(QString type, QString msg) {
	//info, error, standby, and startgame
	QString color = "#000000";
	if (type == "info") color="#000000";
	if (type == "error") color="#A00000";
	if (type == "standby") color="#00A000";
	if (type == "startgame") color="#0000A0";
	ui->events->append("<span style=\"color:"+color+"\">"+msg+"</span>");
	if (!eventVisible) {
		ui->dockEvents->setWindowTitle("*Events*");
	}
}

void Capitalism::setCurGame() {
	int gid = client->getCurGameId();
	if (gid == -1) {
		setGame(0);
		layout->setCurrentIndex(0);
	} else {
		setGame(client->getGame(gid));
		//Set gameconfig as central widget
		if (game->getStatus() == "config") {
			layout->setCurrentIndex(1);
		} else if (game->getStatus() == "run") {
			layout->setCurrentIndex(2);
		}
	}
}

void Capitalism::startGame(int r, int c) {
	if (games == 0) return;
	QTableWidgetItem* it = games->item(r, 3);
	if (client == 0) return;
	client->send(it->text());
}

void Capitalism::addToList(Player* p) {
	ui->playerInfos->addWidget(p->playerWidget);
	ui->playerInfos->update();

	connect(p->playerWidget, SIGNAL(trade(QString)), client, SLOT(send(QString)));
	connect(p->auction, SIGNAL(trade(QString)), client, SLOT(send(QString)));
}

void Capitalism::newGames() {
	games = new QTableWidget();
	games->setObjectName("Lobby");
	games->setHorizontalHeaderLabels(QStringList("Name") << "Description" << "Players");

	connect(games, SIGNAL(cellDoubleClicked(int,int)), this, SLOT(startGame(int, int)));
}

void Capitalism::newGameConfig() {
	gameConfig = new GameConfig();
	gameConfig->setObjectName("Config");
	if (client == 0) return;

	connect(client, SIGNAL(configUpdate(QString,QString,bool,bool)), gameConfig, SLOT(configUpdate(QString, QString, bool, bool)));
	connect(gameConfig, SIGNAL(optionChanged(QString)), client, SLOT(send(QString)));
}

void Capitalism::newField() {
	field = new Field();
	field->setMinimumWidth(500);
	field->setObjectName("Field");

	if (client == 0) return;
	connect(client, SIGNAL(gotText(QString)), field, SLOT(appendLine(QString)));
	connect(client, SIGNAL(clearedText()), field, SLOT(clearText()));
	connect(field, SIGNAL(buttonClicked(QString)), client, SLOT(send(QString)));
	connect(client, SIGNAL(gotButton(QString,QString)), field, SLOT(addButton(QString,QString)));
	connect(client, SIGNAL(playerAdded(Player*)), field, SLOT(addPlayerAuction(Player*)));

	connect(client, SIGNAL(auctionStarted()), field, SLOT(showAuction()));
	connect(client, SIGNAL(auctionIdUpdate(int)), field, SLOT(updatAuctionId(int)));
	connect(client, SIGNAL(auctionFieldUpdate(int)), field, SLOT(updateAuctionField(int)));
	connect(client, SIGNAL(auctionBidUpdate(int,int)), field, SLOT(bidUpdate(int,int)));
	connect(client, SIGNAL(auctionStatusUpdate(int)), field, SLOT(updateAuctionStatus(int)));

	connect(field->auction, SIGNAL(bid(QString)), client, SLOT(send(QString)));

	connect(field, SIGNAL(fieldSelected(MonopdField*)), this, SLOT(setSelectedField(MonopdField*)));
}

Trade* Capitalism::getTrade(int tid) {
	Trade* t = 0;
	for (int i = 0; i < trades.size(); i++)
		if (trades.value(i)->getId() == tid) t = trades.value(i);
	return t;
}

void Capitalism::removeTrade(Trade* t) {
	trades.removeAll(t);
	t->~Trade();
}

void Capitalism::send(QString msg) {
	if (client == 0) return;
	client->send(msg);
}

void Capitalism::drawField() {
	if (field == 0) return;
	if (client->getCurGameId() != -1) {
		MonopdGame* g = client->getGame(client->getCurGameId());
		if (g->getStatus() != "config")
			field->update();
	}
}

void Capitalism::tradeUpdate(QString type, int tradeId, int actor, int rev) {
	if (type == "new") {
		Trade* t = new Trade(tradeId);
		t->setRev(0);
		t->setGame(game);
		if (actor != -1) t->updateTradeActor(actor);
		trades.append(t);
		connect(t, SIGNAL(tradeCommand(QString)), client, SLOT(send(QString)));
		t->show();
	} else if (type == "edit") {
		Trade* t = getTrade(tradeId);
		if (actor != -1) t->updateTradeActor(actor);
		if (t == 0) return;
		if (rev > 0) t->setRev(rev);
	} else if ((type == "rejected") || (type == "completed")) {
		Trade* t = getTrade(tradeId);
		if (t == 0) return;

		removeTrade(t);
	}
}

void Capitalism::tradePlayerUpdate(int tradeId, int playerid, bool accept) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateTradeAccept(playerid, accept);
}

void Capitalism::tradeEstateUpdate(int tradeId, int estateId, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateEstateTrade(estateId, targetPlayerId);
}

void Capitalism::tradeCardUpdate(int tradeId, int cardId, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateCardTrade(cardId, targetPlayerId);
}

void Capitalism::tradeMoneyUpdate(int tradeId, int fromPlayerId, int money, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateMoneyTrade(fromPlayerId, money, targetPlayerId);
}

